package org.bytedream.quizzledizzle.console;

import com.diogonunes.jcolor.Ansi;
import com.diogonunes.jcolor.Attribute;
import org.bytedream.quizzledizzle.question.Question;

import java.util.Scanner;

public class Console {

    public static final boolean isWindows = System.getProperty("os.name").toLowerCase().contains("win");
    private final Scanner scanner = new Scanner(System.in);

    public String ask(String question) {
        System.out.print(question);
        return scanner.next();
    }

    public int askInt(String question) {
        String maybeNum;
        while (true) {
            System.out.print(question);
            maybeNum = scanner.next();

            try {
                return Integer.parseInt(maybeNum);
            } catch (NumberFormatException e) {
                if (maybeNum.equalsIgnoreCase("q")) {
                    return -1;
                } else {
                    System.out.println(Ansi.colorize("Please enter a number or 'q' to exit", Attribute.RED_TEXT()));
                }
            }
        }
    }

    public void title(String content) {
        System.out.println(content);
    }

    public void error(String content) {
        System.out.println(Ansi.colorize(content, Attribute.RED_TEXT()));
    }

    public void info(String content) {
        System.out.println(Ansi.colorize(content, Attribute.BRIGHT_BLUE_TEXT()));
    }

    /**
     * shows a question
     *
     * @param question
     * @return the answer index
     */
    public int showQuestion(Question question) {
        System.out.println(Ansi.colorize(question.getQuestion(), Attribute.BLUE_TEXT()));

        String[] answers = question.getAnswers();
        for (int i = 0; i < answers.length; i++) {
            System.out.printf(" -> %d) %s\n", i + 1, answers[i]);
        }
        while (true) {
            System.out.println("");
            System.out.print("Enter a number: ");
            String answer = scanner.next();
            if (answer.equals("q")) {
                return -1;
            } else {
                try {
                    int chosenAnswer = Integer.parseInt(answer) - 1;
                    if (chosenAnswer >= 0 && chosenAnswer < answers.length) {
                        return chosenAnswer;
                    }
                } catch (NumberFormatException ignore) {}
                clearLastLine();
                clearLastLine();
                System.err.println("Please enter a valid number");
            }
        }
    }

    /**
     * shows the results from a question
     *
     * @param question
     * @param chosenAnswer
     */
    public void showResult(Question question, int chosenAnswer) {
        String[] answers = question.getAnswers();

        char answerEmoji;
        for (int i = 0; i < answers.length; i++) {
            if (i == chosenAnswer) {
                answerEmoji = '✔';
            } else {
                answerEmoji = '✘';
            }
            if (i == question.getRightAnswerIndex()) {
                System.out.println(Ansi.colorize(String.format("»» %d) %s [%s] ««", i + 1, answers[i], answerEmoji), Attribute.GREEN_TEXT()));
            } else {
                System.out.println(Ansi.colorize(String.format("   %d) %s [%s]", i + 1, answers[i], answerEmoji), Attribute.RED_TEXT()));
            }
        }
    }

    /**
     * clears the screen and shows the total questions and the count of right questions
     *
     * @param totalQuestions
     * @param rightQuestions
     */
    public void showClearScreen(int totalQuestions, int rightQuestions) {
        clearScreen();
        System.out.println(Ansi.colorize(String.format("Total questions: %d - Right questions: %d\n", totalQuestions, rightQuestions), Attribute.BLUE_TEXT()));
    }

    /**
     * clears the entire screen.
     * repeats \n if the plaform is windows, ansi codes are used on all other plaforms
     */
    public void clearScreen() {
        if (!isWindows) {
            System.out.print("\033[H\033[2J");
        } else {
            // because windows is windows, ansi codes are not working by default and this has to be used
            System.out.println("\n".repeat(1000));
        }
    }

    /**
     * clears the last line.
     * only works on linux
     */
    public void clearLastLine() {
        // yep, windows again...
        if (!isWindows) {
            System.out.print("\033[1F\33[2K");
        }
    }

}
